// ardutx v1.1
// (c) 2014 patricio reinoso
// www.patolin.com

// v1.0 version final
// v1.1 agregado medidor de voltaje de bateria


#include <LCD5110_Graph.h>
#include <EEPROM.h>

// config PPM
#define servoCentro 1500       //servo center value (us)
#define ppmLong 22500          //ppm frame
#define ppmPulso 300           //ppm pulse width
#define onState 0              //Polarity: 0 neg / 1 pos
#define sigPin 10              //ppm output pin
#define ppmMin 1000
#define ppmMax 2000

// config TX
#define CANALES 8              //number of channels
#define MODELOS 5              //total model memory
#define canalTh 2              //Throtle channel ID
#define potMin 0               //Pot min value
#define potMax 1023            //Por max value

#define txVer "v1.1"

#define numBytesPorModelo 32
#define swTC 12
#define swDR 11

int ppm[CANALES];              //ppm output array
unsigned char sw_status=0x00;  //0:TC 1:DR 
unsigned int pots[CANALES];    //ADC data array
unsigned char inPots[CANALES] = {
  A0, A1, A2, A3, A4, A5};    //Input pins array
unsigned char batPin = A7;

LCD5110 myGLCD(3,4,5,6,7);//Nokia 5100 pins  

// nothing editable from here :)
// *****************************
extern uint8_t copter_logo[]; 

extern unsigned char TinyFont[];
unsigned char inTeclado=A6;

unsigned char menuActual=0;
unsigned char menuSubActual=0;
unsigned char submenuActual=0;
unsigned char drSeleccion=0xFF;
unsigned char pantalla=0;
unsigned char menu=0;
unsigned char modeloActual=0;

unsigned char servoReversa;
unsigned char dual_rate_low[CANALES];
unsigned char dual_rate_hi[CANALES];

boolean displayReady=false;
int temporal=0;
int calibracion[CANALES][2];
char msgs[][19] = {
  "Main Menu",
  "Servo Direction", 
  "Dual Rates", 
  "Model Selection", 
  "Save data", 
  "Stick Calibration"
};

// timer1 setup
void configTimer1() {
  cli();
  TCCR1A = 0;
  TCCR1B = 0;

  OCR1A = 100;
  TCCR1B |= (1 << WGM12);
  TCCR1B |= (1 << CS11);  // prescaler 8: 0.5us - 16mhz
  TIMSK1 |= (1 << OCIE1A);
  sei();  
}

// Interrupcion del timer
ISR(TIMER1_COMPA_vect){  
  static boolean state = true;

  TCNT1 = 0;

  if(state) {   
    digitalWrite(sigPin, onState);
    OCR1A = ppmPulso * 2;
    state = false;

  }
  else{   
    static byte cur_chan_numb;
    static unsigned int calc_rest;

    digitalWrite(sigPin, !onState);
    state = true;

    if(cur_chan_numb >= CANALES){
      cur_chan_numb = 0;
      calc_rest = calc_rest + ppmPulso; 
      OCR1A = (ppmLong - calc_rest) * 2;
      calc_rest = 0;
    }
    else{
      OCR1A = (ppm[cur_chan_numb] - ppmPulso) * 2;
      calc_rest = calc_rest + ppm[cur_chan_numb];
      cur_chan_numb++;
    }     
  }

}


void setup() {
  unsigned char i;
  // switches setuo
  pinMode(swTC, INPUT_PULLUP);
  pinMode(swDR, INPUT_PULLUP);
  digitalWrite(swTC, HIGH); // pullup
  digitalWrite(swDR, HIGH); // pullup

  // zero state values for ppm
  ppm[0]=servoCentro;
  ppm[1]=servoCentro;
  ppm[2]=ppmMin;
  ppm[3]=servoCentro;
  ppm[4]=servoCentro;
  ppm[5]=servoCentro;

  //zero state config parameters

  // servo reverse: 0 normal, 1 reverse
  servoReversa=0b00000000;

  // dual rates por defecto
  for (i=0;i<=CANALES;i++) {
    dual_rate_low[i]=75;
    dual_rate_hi[i]=100;
  }

  // load data from eeprom
  //reseteaEeprom();
  modeloActual=cargaDatosEeprom(255);

  // LCD config
  myGLCD.InitLCD();
  myGLCD.setFont(TinyFont);
  myGLCD.clrScr();
  myGLCD.drawBitmap(0, 0, copter_logo, 84, 48);
  myGLCD.update();
  delay(2000);
  myGLCD.clrScr();
  myGLCD.print("ArduTX", CENTER, 0);
  myGLCD.print(txVer, CENTER, 20);
  myGLCD.drawRoundRect(24, 16, 60, 30);
//  for (int i=0; i<6; i++)
//  {
//    myGLCD.drawLine(57, 18+(i*2), 83-(i*3), 18+(i*2));
//    myGLCD.drawLine((i*3), 28-(i*2), 28, 28-(i*2));
//  }
  myGLCD.setFont(TinyFont);
  myGLCD.print("(c) 2014", CENTER, 36);
  myGLCD.print("www.patolin.com", CENTER, 42);
  myGLCD.update();
  delay(2000);

  // ppm timer config
  pinMode(sigPin, OUTPUT);
  digitalWrite(sigPin, !onState);
  configTimer1();
}

void loop() {
  int i;
  int j=0;
  unsigned char tecla=0;
  int vBat;
  unsigned char valT;
  while(1) {
    // read bat voltage
    vBat=analogRead(batPin);
    
    // read potentiometers
    readPots();
    readSwitches();
    // read keyboard value every 10th loop and store it
    // and set the values depending on the menu option
    displayReady=false;
    myGLCD.clrScr();
    if (pantalla==0) {
      // info screen
      unsigned char valBar;
      myGLCD.print(txVer, 30, 0); 
      myGLCD.print("Mod: ", 0, 40);  
      myGLCD.print(String(modeloActual+1), 20, 40); 
      // throtle cut switch value
      if (bitRead(sw_status,0)==0) {
        myGLCD.print("TC", 30, 40);
      } 
      else {
        myGLCD.print("  ", 30, 40);
      }
      // dual rates switch value
      if (bitRead(sw_status,1)==0) {
        myGLCD.print("Rate: LO", 45, 40);
      } 
      else {
        myGLCD.print("Rate: HI", 45, 40);
      }
      
      // batery Voltage
      myGLCD.print("Bat: ", 45, 33);  
      int voltajeBat=map(vBat,0,1023,0,100);
      myGLCD.print(String(voltajeBat), 65, 33);
      myGLCD.print("%", 75, 33);

      // draw the bars
      myGLCD.drawLine(48,6,48,32);
      for (i=0;i<=3;i++) {
        // level bars
        valBar=pots[i]/16;
        myGLCD.drawRect(15,7+(i*7),81,9+(i*7));

        myGLCD.setPixel(16+valBar,8+(i*7));
        myGLCD.setPixel(15+valBar,8+(i*7));
        myGLCD.setPixel(17+valBar,8+(i*7));

        // servo reverse indicator
        if (bitRead(servoReversa, i)) {
          myGLCD.print("R", 10, 6+(i*7)); 
        } 
        else {
          myGLCD.print("N", 10, 6+(i*7));      
        }

      }
    } 
    else {
      if (pantalla==1) {
        // main menu
        myGLCD.print(msgs[0], 0, 0);
        for (i=1;i<=5;i++) {
          if (i==menuSubActual) {
            myGLCD.print("->", 0, i*7);
            myGLCD.print(msgs[i], 8, i*7);
          } 
          else {
            myGLCD.print("  ", 0, i*7);
            myGLCD.print(msgs[i], 8, i*7);
          }
        }
      }

      if (pantalla==2) {
        if ( menuActual==1) {
          // servo reversing
          myGLCD.print(msgs[menuActual], 0, 0);
          for (i=0;i<=(CANALES-1);i++) {
            if (i==menuSubActual) {
              myGLCD.print("->", 0, 7+i*7);                   
            } 
            else {
              myGLCD.print("  ", 0, 7+i*7);                
            }
            myGLCD.print("CH: ", 8, 7+i*7);
            myGLCD.print(String(i+1), 20, 7+i*7);
            if (bitRead(servoReversa,i)) {
              myGLCD.print(" - R", 26, 7+i*7);
            } 
            else {
              myGLCD.print(" - N", 26, 7+i*7);
            }
          }
          myGLCD.print("Mod: ", 55, 42);  
          myGLCD.print(String(modeloActual+1), 75, 42);
        }

        if(menuActual==2) {
          // dual rates
          unsigned char contadorTemp=0;
          myGLCD.print(msgs[menuActual], 0, 0);
          for (i=0;i<=(CANALES-1);i++) {
            myGLCD.print("CH: ", 0, 7+i*7);
            myGLCD.print(String(i+1), 12, 7+i*7);
            if (menuSubActual==contadorTemp) {
              if (drSeleccion==contadorTemp) {
                myGLCD.print("*", 26, 7+i*7);  
              } 
              else {
                myGLCD.print(">", 26, 7+i*7);
              }
            }

            myGLCD.print("L:", 30, 7+i*7);
            myGLCD.print(String(dual_rate_low[i]), 38, 7+i*7);
            contadorTemp++;
            if (menuSubActual==contadorTemp) {
              if (drSeleccion==contadorTemp) {
                myGLCD.print("*", 56, 7+i*7);  
              } 
              else {
                myGLCD.print(">", 56, 7+i*7);
              }
            }
            myGLCD.print("H:", 60, 7+i*7);
            myGLCD.print(String(dual_rate_hi[i]), 68, 7+i*7);
            contadorTemp++;
          }
        }

        if (menuActual==3) {
          // model selection
          myGLCD.print(msgs[menuActual], 0, 0);
          for (i=0;i<=(MODELOS-1);i++) {
            myGLCD.print("Mod: ", 8, 7+i*7);
            myGLCD.print(String(i+1), 27, 7+i*7);
            if (i==menuSubActual) {
              myGLCD.print("->", 0, 7+i*7);                   
            } 
            else {
              myGLCD.print("  ", 0, 7+i*7);                
            }
            if (i==modeloActual) {
              myGLCD.print("*", 35, 7+i*7);
            }
          }
        }

        if (menuActual==4) {
          // data save
          int eepromBase;
          eepromBase=numBytesPorModelo*modeloActual;
          unsigned char eepromPos =eepromBase;
          // actual model
          EEPROM.write(511,modeloActual);
          // servo reverse
          EEPROM.write(eepromPos, servoReversa);
          eepromPos++;
          // dual rates
          for (i=0;i<=CANALES;i++) {
            EEPROM.write(eepromPos, dual_rate_low[i]);
            eepromPos++;
            EEPROM.write(eepromPos, dual_rate_hi[i]);
            eepromPos++;
          }

          myGLCD.print("Saving data for", 0, 20);
          myGLCD.print("actual model:", 0, 28);
          myGLCD.print(String(modeloActual+1), 0, 36);
          myGLCD.print("...wait", 0, 42);
          pantalla--;
          menuActual=1;
          menuSubActual=1;
          myGLCD.update();
          delay(1000);
        }

        if (menuActual==5) {
          // gimbal calibration
          myGLCD.print(msgs[menuActual], 0, 0);
          myGLCD.update();
          delay(500);
          myGLCD.clrScr();
          for (i=0;i<=CANALES;i++) {
            calibracion[i][0]=512;
            calibracion[i][1]=512;

          }
          tecla=readKeys(); 
          while (tecla!=1) {
            myGLCD.clrScr();
            myGLCD.print("Calibration", 0, 0);
            myGLCD.print("Move the sitcks", 0, 8);

            myGLCD.print("1: ", 0, 16);
            myGLCD.print(String(analogRead(inPots[0])), 20, 16);

            myGLCD.print("2: ", 40, 16);
            myGLCD.print(String(analogRead(inPots[1])), 60, 16);

            myGLCD.print("3: ", 0, 24);
            myGLCD.print(String(analogRead(inPots[2])), 20, 24);

            myGLCD.print("4: ", 40, 24);
            myGLCD.print(String(analogRead(inPots[3])), 60, 24);

            myGLCD.print("5: ", 0, 32);
            myGLCD.print(String(analogRead(inPots[4])), 20, 32);

            myGLCD.print("6: ", 40, 32);
            myGLCD.print(String(analogRead(inPots[5])), 60, 32);

            myGLCD.update();
            int lecturaTemporal;
            for (i=0;i<=(CANALES-1);i++) {
              lecturaTemporal=analogRead(inPots[i]);
              // min values
              if (lecturaTemporal<=calibracion[i][0]) {
                calibracion[i][0]=lecturaTemporal;
              }
              // max values
              if (lecturaTemporal>=calibracion[i][1]) {
                calibracion[i][1]=lecturaTemporal;
              }
              delay(10);
            }
            tecla=readKeys();
          }
          // save calibration data
          int posEeprom=0;
          for (i=0;i<=(CANALES-1);i++) {
            posEeprom=400+(i*4);
            EEPROMWriteInt(posEeprom, calibracion[i][0]); //min
            posEeprom+=2;
            EEPROMWriteInt(posEeprom, calibracion[i][1]); //max
          }
          myGLCD.clrScr();
          myGLCD.print("Saving calibration", 0, 20);
          myGLCD.print("data", 0, 28);
          myGLCD.print("...wait", 0, 40);
          myGLCD.update();
          delay(1000);
          pantalla=0;
          menuActual=0;
          menuSubActual=0;
        }
      }

    }

    // screen update
    myGLCD.update();


    // read keyboard every 2 loops
    if (j>2) {
      tecla=readKeys(); 
      if (tecla==2) {
        if (pantalla<2) { 
          pantalla++; 
          menuActual=menuSubActual; 
          menuSubActual=0;
        } 
        else {
          // options for each menu
          if (menuActual==1) {
            // reversa de servos
            bitWrite(servoReversa, menuSubActual, !bitRead(servoReversa,menuSubActual));
          }
          if (menuActual==2) {
            // dual rates
            if (drSeleccion!=0xFF) {
              drSeleccion=0xFF;
            } 
            else {
              drSeleccion=menuSubActual;
            }
          }

          if (menuActual==3) {
            // model selection
            modeloActual=menuSubActual;
            // load data from eeprom
            EEPROM.write(511, modeloActual);
            cargaDatosEeprom(modeloActual);
            pantalla=0; 
            menuSubActual=0; 
            menuActual=0;
          }
        }
      }

      if (tecla==3) {
        switch (menuActual) {
        case 2:
          if (drSeleccion==0xFF) {
            if (menuSubActual<(CANALES*2)) { 
              menuSubActual++; 
              if (pantalla==0) {
                pantalla++; 
              }
            }
          } 
          else {
            if ((drSeleccion%2)==0) {
              if (dual_rate_low[drSeleccion/2]>0) { 
                dual_rate_low[drSeleccion/2]--;  
              }    
            } 
            else {
              if (dual_rate_hi[drSeleccion/2]>0) { 
                dual_rate_hi[drSeleccion/2]--;  
              } 
            }
          }
          break;
        default: 
          if (menuSubActual<6) { 
            menuSubActual++; 
            if (pantalla==0) {
              pantalla++; 
            }
          }
          break;
        }   

      }

      if (tecla==1) {
        switch (menuActual) {
        case 2:
          if (drSeleccion==0xFF) {
            if (menuSubActual==0) { 
              pantalla--; 
              menuSubActual=1; 
              menuActual=0;
            } 
            else { 
              menuSubActual--; 
            } 
          } 
          else {
            if ((drSeleccion%2)==0) {
              if (dual_rate_low[drSeleccion/2]<100) { 
                dual_rate_low[drSeleccion/2]++;  
              }
            } 
            else {
              if (dual_rate_hi[drSeleccion/2]<100) { 
                dual_rate_hi[drSeleccion/2]++;  
              } 
            }
          }
          break;
        default:
          if (menuSubActual>0) {
            menuSubActual--; 
          } 
          else {
            if (pantalla>0) { 
              pantalla--; 
              menuSubActual=1; 
              menuActual=0;
            }
          } 
          break;
        }
      }

      j=0;
    }
    j++;

  }
}

void readPots() {
  unsigned char i,j;
  int temporalPots[CANALES];
  for (i=0;i<=(CANALES-1);i++) {
    // read the pots     
    temporalPots[i]=analogRead(inPots[i]);
    // calibration mapping
    pots[i]=map(temporalPots[i],calibracion[i][0], calibracion[i][1], 0, 1023);
    // servo reversing
    if (bitRead(servoReversa,i)==1) {
      pots[i]=1023-pots[i]; 
    }

    // Throtle cut
    if (i==canalTh && bitRead(sw_status,0)==0) {
      // TC activado
      pots[i]=0;
    }

    // dual rates
    if (bitRead(sw_status,1)==0) {
      // low, rates
      pots[i]=ajusteDR(pots[i],dual_rate_low[i],i,canalTh);
    } 
    else {
      // hi rates 
      pots[i]=ajusteDR(pots[i],dual_rate_hi[i],i,canalTh);
    }  
    // map values to PPM
    ppm[i]=map(pots[i],potMin, potMax, ppmMin, ppmMax);
  }
}

void readSwitches() {
  // read switches and set into the sw_status var
  if (digitalRead(swTC)==HIGH) {
    bitWrite(sw_status,0,1);
  } 
  else {
    bitWrite(sw_status,0,0);
  }
  if (digitalRead(swDR)==HIGH) {
    bitWrite(sw_status,1,1);
  } 
  else {
    bitWrite(sw_status,1,0);
  }
}

unsigned char readKeys() {
  // read analog keyboard    // 933 523  941 244
  unsigned char salida;  
  int valTeclado = analogRead(inTeclado);
  delay(25);
  //int valTemp=analogRead(inTeclado)-valTeclado;
  //if (10<abs(valTemp)) salida=0;
  if (valTeclado>=960) salida=0;
  else if (valTeclado>910 && valTeclado<950) salida=3;
  else if (valTeclado>250 && valTeclado<540) salida=1;
  else if (valTeclado>100 && valTeclado<245) salida=2;
  return salida;
}

int  freeRam () {
  // returns free ram in avr chip
  extern int __heap_start, *__brkval; 
  int v; 
  return (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval); 
}

unsigned char cargaDatosEeprom(unsigned char mod) {
  // load data from eeprom, for selected model
  int eepromBase;
  unsigned char i;
  if (mod=255) {
    mod=EEPROM.read(511);
  } 

  eepromBase=numBytesPorModelo*mod;
  unsigned char eepromPos=eepromBase;
  // servo reversing
  servoReversa=EEPROM.read(eepromPos);
  eepromPos++;
  // dual rates
  for (i=0;i<=CANALES-1;i++) {
    dual_rate_low[i]=EEPROM.read(eepromPos);
    eepromPos++;
    dual_rate_hi[i]=EEPROM.read(eepromPos);
    eepromPos++;
  }
  int posEeprom;
  for (i=0;i<=(CANALES-1);i++) {
    posEeprom=400+(i*4);
    calibracion[i][0]=EEPROMReadInt(posEeprom); //minimo
    posEeprom+=2;
    calibracion[i][1]=EEPROMReadInt(posEeprom); //maximo
  }

  return mod;
}

unsigned char reseteaEeprom() {
  // erases eeprom, just for checking values (unused now)
  int eepromBase, eepromPos;
  unsigned char i,j;
  EEPROM.write(511,0);
  for (j=0;j<=CANALES-1;j++) {
    eepromPos=numBytesPorModelo*j;
    EEPROM.write(eepromPos,0x00);
    eepromPos++;
    for (i=0;i<=CANALES-1;i++) {
      EEPROM.write(eepromPos,75);
      eepromPos++;
      EEPROM.write(eepromPos,100);
      eepromPos++;

    }
  }      
}


int ajusteDR(int valor, float dr, unsigned char canal, unsigned char canTh) {
  // dual rate calc
  int salida,temp;
  float mult;
  mult=dr;
  if (canal==canalTh) {
    salida=(int)valor*mult/100;  
  } 
  else {
    temp=valor-512;
    salida=(int)((temp*mult/100)+512);

  }
  salida=constrain(salida,0,1023);
  return salida;
}


void EEPROMWriteInt(int p_address, int p_value) {
  // write a 16bit value in eeprom
  byte Byte1 = ((p_value >> 0) & 0xFF);
  byte Byte2 = ((p_value >> 8) & 0xFF);
  EEPROM.write(p_address, Byte1);
  EEPROM.write(p_address + 1, Byte2);
}

int EEPROMReadInt(int p_address) {
  // read a 16 bit value in eeprom
  byte Byte1 = EEPROM.read(p_address);
  byte Byte2 = EEPROM.read(p_address + 1);
  long firstTwoBytes = ((Byte1 << 0) & 0xFF) + ((Byte2 << 8) & 0xFF00);
  return (firstTwoBytes);
}

